@@ -1,37 +1,37 @@
 {-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE QuasiQuotes #-}
-{-# LANGUAGE TemplateHaskell #-}
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings  #-}
 
-import           Control.Exception            (Exception)
+import           Control.Exception            (Exception, toException)
 import           Control.Monad.IO.Class       (liftIO)
-import           Data.Typeable                (Typeable)
-import           Data.XML.Types
-import           Test.HUnit                   hiding (Test)
-import           Test.Hspec
 import qualified Data.ByteString.Char8        as S
 import qualified Data.ByteString.Lazy.Char8   as L
-import qualified Text.XML.Unresolved          as D
-import qualified Text.XML.Stream.Parse        as P
+import           Data.Typeable                (Typeable)
+import           Data.XML.Types
+import           Test.Hspec
+import           Test.HUnit                   hiding (Test)
 import qualified Text.XML                     as Res
 import qualified Text.XML.Cursor              as Cu
 import           Text.XML.Stream.Parse        (def)
+import qualified Text.XML.Stream.Parse        as P
+import qualified Text.XML.Unresolved          as D
 
-import Text.XML.Cursor ((&/), (&//), (&.//), ($|), ($/), ($//), ($.//))
-import Data.Text(Text)
-import Control.Monad
-import Control.Applicative ((<$>))
-import Control.Monad.Trans.Class (lift)
-import qualified Data.Text as T
-import qualified Data.Set as Set
-import Control.Exception (toException)
+import           Control.Applicative          ((<$>))
+import           Control.Monad
+import           Control.Monad.Trans.Class    (lift)
+import qualified Data.Set                     as Set
+import           Data.Text                    (Text)
+import qualified Data.Text                    as T
+import           Text.XML.Cursor              (($.//), ($/), ($//), ($|),
+                                               (&.//), (&/), (&//))
 
-import qualified Data.Conduit as C
+import           Control.Monad.Trans.Resource (runResourceT)
 import qualified Control.Monad.Trans.Resource as C
-import qualified Data.Conduit.List as CL
-import qualified Data.Map as Map
-import Text.Blaze (toMarkup)
-import Text.Blaze.Renderer.String (renderMarkup)
+import           Data.Conduit                 ((=$=))
+import qualified Data.Conduit                 as C
+import qualified Data.Conduit.List            as CL
+import qualified Data.Map                     as Map
+import           Text.Blaze                   (toMarkup)
+import           Text.Blaze.Renderer.String   (renderMarkup)
 
 main :: IO ()
 main = hspec $ do
@@ -42,12 +42,16 @@ main = hspec $ do
         it "has working many function" testMany
         it "has working many' function" testMany'
         it "has working manyYield function" testManyYield
+        it "has working takeContent function" testTakeContent
+        it "has working takeTree function" testTakeTree
+        it "has working takeAnyTreeContent function" testTakeAnyTreeContent
         it "has working orE" testOrE
         it "is idempotent to parse and pretty render a document" documentParsePrettyRender
         it "ignores the BOM" parseIgnoreBOM
         it "strips duplicated attributes" stripDuplicateAttributes
         it "displays comments" testRenderComments
         it "conduit parser" testConduitParser
+        it "can omit the XML declaration" omitXMLDeclaration
     describe "XML Cursors" $ do
         it "has correct parent" cursorParent
         it "has correct ancestor" cursorAncestor
@@ -94,7 +98,7 @@ main = hspec $ do
     it "parsing CDATA" caseParseCdata
     it "retains namespaces when asked" caseRetainNamespaces
     it "handles iso-8859-1" caseIso8859_1
-    it "renders CDATA when asked" caseRenderCDATA 
+    it "renders CDATA when asked" caseRenderCDATA
     it "escapes CDATA closing tag in CDATA" caseEscapesCDATA
 
 documentParseRender :: IO ()
@@ -134,8 +138,8 @@ documentParsePrettyRender =
         ]
 
 combinators :: Assertion
-combinators = C.runResourceT $ P.parseLBS def input C.$$ do
-    P.force "need hello" $ P.tagName "hello" (P.requireAttr "world") $ \world -> do
+combinators = runResourceT $ P.parseLBS def input C.$$ do
+    P.force "need hello" $ P.tag' "hello" (P.requireAttr "world") $ \world -> do
         liftIO $ world @?= "true"
         P.force "need child1" $ P.tagNoAttr "{mynamespace}child1" $ return ()
         P.force "need child2" $ P.tagNoAttr "child2" $ return ()
@@ -181,7 +185,7 @@ testChoose = do
         testChooseElemOrTextIsChunkedText2
 
 testChooseElemOrTextIsText :: Assertion
-testChooseElemOrTextIsText = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseElemOrTextIsText = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.tagNoAttr "failure" $ return "boom"
@@ -198,7 +202,7 @@ testChooseElemOrTextIsText = C.runResourceT $ P.parseLBS def input C.$$ do
         ]
 
 testChooseElemOrTextIsEncoded :: Assertion
-testChooseElemOrTextIsEncoded = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseElemOrTextIsEncoded = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.tagNoAttr "failure" $ return "boom"
@@ -215,7 +219,7 @@ testChooseElemOrTextIsEncoded = C.runResourceT $ P.parseLBS def input C.$$ do
         ]
 
 testChooseElemOrTextIsEncodedNBSP :: Assertion
-testChooseElemOrTextIsEncodedNBSP = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseElemOrTextIsEncodedNBSP = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.tagNoAttr "failure" $ return "boom"
@@ -233,7 +237,7 @@ testChooseElemOrTextIsEncodedNBSP = C.runResourceT $ P.parseLBS def input C.$$ d
 
 
 testChooseElemOrTextIsWhiteSpace :: Assertion
-testChooseElemOrTextIsWhiteSpace = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseElemOrTextIsWhiteSpace = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.tagNoAttr "failure" $ return "boom"
@@ -248,7 +252,7 @@ testChooseElemOrTextIsWhiteSpace = C.runResourceT $ P.parseLBS def input C.$$ do
         ]
 
 testChooseTextOrElemIsWhiteSpace :: Assertion
-testChooseTextOrElemIsWhiteSpace = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseTextOrElemIsWhiteSpace = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.contentMaybe
@@ -263,7 +267,7 @@ testChooseTextOrElemIsWhiteSpace = C.runResourceT $ P.parseLBS def input C.$$ do
         ]
 
 testChooseElemOrTextIsChunkedText :: Assertion
-testChooseElemOrTextIsChunkedText = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseElemOrTextIsChunkedText = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.tagNoAttr "failure" $ return "boom"
@@ -278,7 +282,7 @@ testChooseElemOrTextIsChunkedText = C.runResourceT $ P.parseLBS def input C.$$ d
         ]
 
 testChooseElemOrTextIsChunkedText2 :: Assertion
-testChooseElemOrTextIsChunkedText2 = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseElemOrTextIsChunkedText2 = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.tagNoAttr "failure" $ return "boom"
@@ -293,7 +297,7 @@ testChooseElemOrTextIsChunkedText2 = C.runResourceT $ P.parseLBS def input C.$$
         ]
 
 testChooseElemOrTextIsElem :: Assertion
-testChooseElemOrTextIsElem = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseElemOrTextIsElem = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.tagNoAttr "success" $ return "success"
@@ -310,7 +314,7 @@ testChooseElemOrTextIsElem = C.runResourceT $ P.parseLBS def input C.$$ do
         ]
 
 testChooseTextOrElemIsText :: Assertion
-testChooseTextOrElemIsText = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseTextOrElemIsText = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.contentMaybe
@@ -327,7 +331,7 @@ testChooseTextOrElemIsText = C.runResourceT $ P.parseLBS def input C.$$ do
         ]
 
 testChooseTextOrElemIsElem :: Assertion
-testChooseTextOrElemIsElem = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseTextOrElemIsElem = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.contentMaybe
@@ -344,7 +348,7 @@ testChooseTextOrElemIsElem = C.runResourceT $ P.parseLBS def input C.$$ do
         ]
 
 testChooseEitherElem :: Assertion
-testChooseEitherElem = C.runResourceT $ P.parseLBS def input C.$$ do
+testChooseEitherElem = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.choose
             [ P.tagNoAttr "failure" $ return 1
@@ -363,9 +367,9 @@ testChooseEitherElem = C.runResourceT $ P.parseLBS def input C.$$ do
 testManyYield :: Assertion
 testManyYield = do
     -- Basically the same as testMany, but consume the streamed result
-    result <- C.runResourceT $
+    result <- runResourceT $
         P.parseLBS def input C.$$ helloParser
-        C.$= CL.consume
+        =$= CL.consume
     length result @?= 5
   where
     helloParser = void $ P.tagNoAttr "hello" $ P.manyYield successParser
@@ -382,8 +386,72 @@ testManyYield = do
         , "</hello>"
         ]
 
+testTakeContent :: Assertion
+testTakeContent = do
+    result <- runResourceT $ P.parseLBS def input C.$$ rootParser
+    result @?= Just
+      [ EventContent (ContentText "Hello world !")
+      ]
+  where
+    rootParser = P.tagNoAttr "root" $ void (P.takeContent >> P.takeContent) =$= CL.consume
+    input = L.concat
+        [ "<?xml version='1.0'?>"
+        , "<!DOCTYPE foo []>\n"
+        , "<root>"
+        , "Hello world !"
+        , "</root>"
+        ]
+
+testTakeTree :: Assertion
+testTakeTree = do
+    result <- runResourceT $ P.parseLBS def input C.$$ rootParser
+    result @?=
+      [ EventBeginDocument
+      , EventBeginDoctype "foo" Nothing
+      , EventEndDoctype
+      , EventBeginElement "a" []
+      , EventBeginElement "em" []
+      , EventContent (ContentText "Hello world !")
+      , EventEndElement "em"
+      , EventEndElement "a"
+      ]
+  where
+    rootParser = void (P.takeTree "a" P.ignoreAttrs) =$= CL.consume
+    input = L.concat
+        [ "<?xml version='1.0'?>"
+        , "<!DOCTYPE foo []>\n"
+        , "<a>"
+        , "<em>Hello world !</em>"
+        , "</a>"
+        , "<b>"
+        , "</b>"
+        ]
+
+testTakeAnyTreeContent :: Assertion
+testTakeAnyTreeContent = do
+    result <- runResourceT $ P.parseLBS def input C.$$ rootParser
+    result @?= Just
+      [ EventBeginElement "b" []
+      , EventContent (ContentText "Hello ")
+      , EventBeginElement "em" []
+      , EventContent (ContentText "world")
+      , EventEndElement "em"
+      , EventContent (ContentText " !")
+      , EventEndElement "b"
+      ]
+  where
+    rootParser = P.tagNoAttr "root" $ (P.takeAnyTreeContent >> void P.ignoreAnyTreeContent) =$= CL.consume
+    input = L.concat
+        [ "<?xml version='1.0'?>"
+        , "<!DOCTYPE foo []>\n"
+        , "<root>"
+        , "<b>Hello <em>world</em> !</b> Welcome !"
+        , "</root>"
+        ]
+
+
 testMany :: Assertion
-testMany = C.runResourceT $ P.parseLBS def input C.$$ do
+testMany = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.many $ P.tagNoAttr "success" $ return ()
         liftIO $ length x @?= 5
@@ -401,7 +469,7 @@ testMany = C.runResourceT $ P.parseLBS def input C.$$ do
         ]
 
 testMany' :: Assertion
-testMany' = C.runResourceT $ P.parseLBS def input C.$$ do
+testMany' = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.many' $ P.tagNoAttr "success" $ return ()
         liftIO $ length x @?= 5
@@ -421,25 +489,29 @@ testMany' = C.runResourceT $ P.parseLBS def input C.$$ do
         ]
 
 testOrE :: IO ()
-testOrE = C.runResourceT $ P.parseLBS def input C.$$ do
+testOrE = runResourceT $ P.parseLBS def input C.$$ do
     P.force "need hello" $ P.tagNoAttr "hello" $ do
         x <- P.tagNoAttr "failure" (return 1) `P.orE`
              P.tagNoAttr "success" (return 2)
+        y <- P.tag' "success" (P.requireAttr "failure") (const $ return 1) `P.orE`
+             P.tag' "success" (P.requireAttr "success") (const $ return 2)
         liftIO $ x @?= Just (2 :: Int)
+        liftIO $ y @?= Just (2 :: Int)
   where
     input = L.concat
         [ "<?xml version='1.0'?>"
         , "<!DOCTYPE foo []>\n"
         , "<hello>"
         , "<success/>"
+        , "<success success=\"0\"/>"
         , "</hello>"
         ]
 
 testConduitParser :: Assertion
-testConduitParser = C.runResourceT $ do
+testConduitParser = runResourceT $ do
     x <- P.parseLBS def input
-        C.$= (P.force "need hello" $ P.tagNoAttr "hello" f)
-        C.$$ CL.consume
+        C.$$ (P.force "need hello" $ P.tagNoAttr "hello" f)
+        =$= CL.consume
     liftIO $ x @?= [1, 1, 1]
   where
     input = L.concat
@@ -456,6 +528,14 @@ testConduitParser = C.runResourceT $ do
         ma <- P.tagNoAttr "item" (return 1)
         maybe (return ()) (\a -> C.yield a >> f) ma
 
+omitXMLDeclaration :: Assertion
+omitXMLDeclaration = Res.renderLBS settings input @?= spec
+  where
+    settings = def { Res.rsXMLDeclaration = False }
+    input = Res.Document (Prologue [] Nothing [])
+              (Res.Element "foo" Map.empty [Res.NodeContent "bar"])
+              []
+    spec = "<foo>bar</foo>"
 
 name :: [Cu.Cursor] -> [Text]
 name [] = []
@@ -736,7 +816,7 @@ caseAttrReorder = do
         rs = def { Res.rsAttrOrder = \name m ->
                         case name of
                             "foo" -> reverse $ Map.toAscList m
-                            _ -> Map.toAscList m
+                            _     -> Map.toAscList m
                  }
         attrs = Map.fromList [("a", "a"), ("b", "b"), ("c", "c")]
         doc = Res.Document (Res.Prologue [] Nothing [])
